import base64
import json
import logging
import os

LOG = logging.getLogger(__name__)
_INFO_STORE = None


class UnableToManageStore(Exception):
    pass


class InfoStore(object):
    def __init__(self, path=None):
        if path is None:
            path = os.path.expanduser(os.path.join('~', '.uvcclient'))
        self._path = path
        self.load()

    def load(self):
        try:
            with open(self._path, 'r') as f:
                self._data = json.loads(base64.b64decode(f.read()).decode())
        except (OSError, IOError):
            LOG.debug('No info store')
            self._data = {}
        except Exception as ex:
            LOG.error('Failed to read store data: %s', ex)
            raise UnableToManageStore('Unable to write to store')

    def save(self):
        try:
            with open(self._path, 'w') as f:
                f.write(
                    base64.b64encode(json.dumps(self._data).encode()).decode())
            os.chmod(self._path, 0o600)
        except (OSError, IOError) as ex:
            LOG.error('Unable to write store: %s', str(ex))
            raise UnableToManageStore('Unable to write to store')

    def get_camera_passwords(self):
        return self._data.get('camera_passwords', {})

    def get_camera_password(self, uuid):
        return self.get_camera_passwords().get(uuid)

    def set_camera_password(self, uuid, password):
        if 'camera_passwords' not in self._data:
            self._data['camera_passwords'] = {}
        self._data['camera_passwords'][uuid] = password
        self.save()


def get_info_store(path=None):
    global _INFO_STORE
    if _INFO_STORE is None:
        _INFO_STORE = InfoStore(path)
    return _INFO_STORE



import contextlib
import mock
import unittest

from uvcclient import store

try:
    import __builtin__ as builtins
except ImportError:
    import builtins


class OpenHelper(object):
    def __init__(self):
        self.mock = mock.MagicMock()

    @contextlib.contextmanager
    def fake_open(self, path, mode):
        self.path = path
        self.mode = mode
        yield self.mock


class TestStore(unittest.TestCase):
    @mock.patch.object(builtins, 'open')
    @mock.patch('os.path.expanduser')
    def test_loads_correct_file_default(self, mock_expand, mock_open):
        mock_expand.return_value = 'foobar'
        mock_open.side_effect = OSError
        store.InfoStore()
        mock_open.assert_called_once_with('foobar', 'r')

    @mock.patch.object(builtins, 'open')
    @mock.patch('os.path.expanduser')
    def test_loads_correct_file(self, mock_expand, mock_open):
        mock_open.side_effect = OSError
        store.InfoStore('barfoo')
        self.assertFalse(mock_expand.called)
        mock_open.assert_called_once_with('barfoo', 'r')

    @mock.patch('os.chmod')
    def test_writes_correct_file(self, mock_chmod):
        with mock.patch.object(builtins, 'open') as mock_open:
            mock_open.side_effect = OSError
            s = store.InfoStore('barfoo')
        opener = OpenHelper()
        with mock.patch.object(builtins, 'open', new=opener.fake_open):
            s.save()
        self.assertTrue(opener.mock.write.called)
        mock_chmod.assert_called_once_with('barfoo', 0o600)

    def test_get_camera_passwords(self):
        with mock.patch.object(builtins, 'open') as mock_open:
            mock_open.side_effect = OSError
            s = store.InfoStore('barfoo')
        s._data = {'camera_passwords': {'foo': 'bar'}}
        self.assertEqual({'foo': 'bar'}, s.get_camera_passwords())
        self.assertEqual('bar', s.get_camera_password('foo'))
        self.assertEqual(None, s.get_camera_password('doesnotexist'))

    def test_set_camera_password(self):
        with mock.patch.object(builtins, 'open') as mock_open:
            mock_open.side_effect = OSError
            s = store.InfoStore('barfoo')
        with mock.patch.object(s, 'save') as mock_save:
            s.set_camera_password('foo', 'bar')
            mock_save.assert_called_once_with()
        self.assertEqual({'foo': 'bar'}, s.get_camera_passwords())



try:
    import httplib
except ImportError:
    from http import client as httplib

import json
import unittest
import zlib

import mock

from uvcclient import nvr


class TestClientLowLevel(unittest.TestCase):
    def setUp(self):
        super(TestClientLowLevel, self).setUp()
        self._patches = []
        try:
            import httplib
            http_mock = mock.patch('httplib.HTTPConnection')
        except ImportError:
            http_mock = mock.patch('http.client.HTTPConnection')
        http_mock.start()
        self._patches.append(http_mock)
        bootstrap_mock = mock.patch.object(nvr.UVCRemote, '_get_bootstrap',
                                           side_effect=self._bootstrap)
        bootstrap_mock.start()
        self._patches.append(bootstrap_mock)

    def _bootstrap(self):
        return {'systemInfo': {'version': '3.1.3'}}

    def cleanUp(self):
        for i in self._patches:
            i.stop()

    def test_uvc_request_get(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 200
        resp.read.return_value = json.dumps({}).encode()
        client._uvc_request('/bar')
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate, sdch',
        }
        conn.request.assert_called_once_with('GET', '/bar?apiKey=key',
                                             None, headers)

    def test_uvc_request_put(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 200
        resp.read.return_value = json.dumps({}).encode()
        result = client._uvc_request('/bar?foo=bar', method='PUT',
                                     data='foobar')
        self.assertEqual({}, result)
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate, sdch',
        }
        conn.request.assert_called_once_with('PUT', '/bar?foo=bar&apiKey=key',
                                             'foobar', headers)

    def test_uvc_request_failed(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 404
        self.assertRaises(nvr.NvrError,
                          client._uvc_request, '/bar', method='PUT',
                          data='foobar')

    def test_uvc_request_failed_noauth(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 401
        self.assertRaises(nvr.NotAuthorized,
                          client._uvc_request, '/bar', method='PUT',
                          data='foobar')

    def test_uvc_request_deflated(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        conn = httplib.HTTPConnection.return_value
        resp = conn.getresponse.return_value
        resp.status = 200
        resp.read.return_value = zlib.compress(json.dumps({}).encode())
        resp.getheaders.return_value = [('Content-Encoding', 'gzip')]
        client._uvc_request('/bar')
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate, sdch',
        }
        conn.request.assert_called_once_with('GET', '/bar?apiKey=key',
                                             None, headers)


class TestClient32(unittest.TestCase):
    @mock.patch.object(nvr.UVCRemote, '_get_bootstrap')
    def test_bootstrap_server_version(self, mock_bootstrap):
        mock_bootstrap.return_value = {'systemInfo': {'version': '3.4.5'}}
        client = nvr.UVCRemote('foo', 7080, 'key')
        self.assertEqual((3, 4, 5), client.server_version)

    @mock.patch.object(nvr.UVCRemote, '_get_bootstrap')
    def test_bootstrap_server_version(self, mock_bootstrap):
        mock_bootstrap.return_value = {'systemInfo': {'version': '3.4.beta5'}}
        client = nvr.UVCRemote('foo', 7080, 'key')
        self.assertEqual((3, 4, 0), client.server_version)

    @mock.patch.object(nvr.UVCRemote, '_get_bootstrap')
    @mock.patch.object(nvr.UVCRemote, 'index')
    def test_310_returns_uuid(self, mock_index, mock_bootstrap):
        mock_index.return_value = [{
            'name': mock.sentinel.name,
            'uuid': mock.sentinel.uuid,
            'id': mock.sentinel.id,
        }]
        mock_bootstrap.return_value = {'systemInfo': {'version': '3.1.0'}}
        client = nvr.UVCRemote('foo', 7080, 'key')
        self.assertEqual(mock.sentinel.uuid, client.name_to_uuid(
            mock.sentinel.name))

    @mock.patch.object(nvr.UVCRemote, '_get_bootstrap')
    @mock.patch.object(nvr.UVCRemote, 'index')
    def test_320_returns_uuid(self, mock_index, mock_bootstrap):
        mock_index.return_value = [{
            'name': mock.sentinel.name,
            'uuid': mock.sentinel.uuid,
            'id': mock.sentinel.id,
        }]
        mock_bootstrap.return_value = {'systemInfo': {'version': '3.2.0'}}
        client = nvr.UVCRemote('foo', 7080, 'key')
        self.assertEqual(mock.sentinel.id, client.name_to_uuid(
            mock.sentinel.name))


class TestClient(unittest.TestCase):
    def setUp(self):
        super(TestClient, self).setUp()
        self._patches = []
        try:
            import httplib
            http_mock = mock.patch('httplib.HTTPConnection')
        except ImportError:
            http_mock = mock.patch('http.client.HTTPConnection')
        http_mock.start()
        self._patches.append(http_mock)
        bootstrap_mock = mock.patch.object(nvr.UVCRemote, '_get_bootstrap',
                                           side_effect=self._bootstrap)
        bootstrap_mock.start()
        self._patches.append(bootstrap_mock)

    def _bootstrap(self):
        return {'systemInfo': {'version': '3.1.3'}}

    def cleanUp(self):
        for i in self._patches:
            i.stop()

    def test_set_recordmode(self):
        fake_resp1 = {'data': [{'recordingSettings': {
            'fullTimeRecordEnabled': False,
            'motionRecordEnabled': False,
        }}]}
        fake_resp2 = {'data': [{'recordingSettings': {
            'fullTimeRecordEnabled': True,
            'motionRecordEnabled': False,
            'channel': 1,
        }}]}

        def fake_req(path, method='GET', data=None):
            if method == 'GET':
                return fake_resp1
            elif method == 'PUT':
                self.assertEqual(json.dumps(fake_resp2['data'][0]), data)
                return fake_resp2

        client = nvr.UVCRemote('foo', 7080, 'key')
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.side_effect = fake_req
            client.set_recordmode('uuid', 'full', chan='medium')
            self.assertTrue(mock_r.called)

        fake_resp2['data'][0]['recordingSettings'] = {
            'fullTimeRecordEnabled': False,
            'motionRecordEnabled': True,
            'channel': 0,
        }
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.side_effect = fake_req
            client.set_recordmode('uuid', 'motion', chan='high')
            self.assertTrue(mock_r.called)

    def test_get_picture_settings(self):
        fake_resp = {'data': [{'ispSettings': {'settingA': 1,
                                               'settingB': 'foo'}}]}
        client = nvr.UVCRemote('foo', 7080, 'key')
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.return_value = fake_resp
            self.assertEqual({'settingA': 1, 'settingB': 'foo'},
                             client.get_picture_settings('uuid'))

    def test_set_picture_settings(self):
        fake_resp = {'data': [{'ispSettings': {'settingA': 1,
                                               'settingB': 'foo'}}]}
        client = nvr.UVCRemote('foo', 7080, 'key')
        newvals = {'settingA': 2, 'settingB': 'foo'}
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.return_value = fake_resp
            resp = client.set_picture_settings('uuid', newvals)
            mock_r.assert_any_call('/api/2.0/camera/uuid', 'PUT',
                                   json.dumps({'ispSettings': newvals}))
            self.assertEqual(fake_resp['data'][0]['ispSettings'], resp)

    def test_set_picture_settings_coerces(self):
        fake_resp = {'data': [{'ispSettings': {'settingA': 1,
                                               'settingB': 'foo',
                                               'settingC': False,
                                           }}]}
        client = nvr.UVCRemote('foo', 7080, 'key')
        newvals = {'settingA': '2', 'settingB': False, 'settingC': 'foo'}
        newvals_expected = {'settingA': 2, 'settingB':
                            'False', 'settingC': True}
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.return_value = fake_resp
            resp = client.set_picture_settings('uuid', newvals)
            mock_r.assert_any_call('/api/2.0/camera/uuid', 'PUT',
                                   json.dumps(
                                       {'ispSettings': newvals_expected}))
            self.assertEqual(fake_resp['data'][0]['ispSettings'], resp)

    def test_get_zones(self):
        fake_resp = {'data': [{'zones': ['fake-zone1',
                                         'fake-zone2']}]}
        client = nvr.UVCRemote('foo', 7080, 'key')
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.return_value = fake_resp
            resp = client.list_zones('uuid')
            mock_r.assert_any_call('/api/2.0/camera/uuid')
            self.assertEqual(fake_resp['data'][0]['zones'], resp)

    def test_prune_zones(self):
        fake_resp = {'data': [{'zones': ['fake-zone1',
                                         'fake-zone2']}]}
        client = nvr.UVCRemote('foo', 7080, 'key')
        with mock.patch.object(client, '_uvc_request') as mock_r:
            mock_r.return_value = fake_resp
            resp = client.prune_zones('uuid')
            mock_r.assert_any_call('/api/2.0/camera/uuid', 'PUT',
                                   json.dumps({'zones': ['fake-zone1']}))

    def test_get_snapshot(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        with mock.patch.object(client, '_safe_request') as mock_r:
            mock_r.return_value.status = 200
            mock_r.return_value.read.return_value = 'image'
            resp = client.get_snapshot('foo')
            mock_r.assert_called_once_with(
                'GET', '/api/2.0/snapshot/camera/foo?force=true&apiKey=key')
            self.assertEqual('image', resp)

    def test_get_snapshot_error(self):
        client = nvr.UVCRemote('foo', 7080, 'key')
        with mock.patch.object(client, '_safe_request') as mock_r:
            mock_r.return_value.status = 401
            self.assertRaises(nvr.NvrError, client.get_snapshot, 'foo')



import unittest

import mock

from uvcclient import nvr


class TestCliUtils(unittest.TestCase):
    FAKE_ENV1 = {
        'UVC': 'http://192.168.1.1:7080/?apiKey=foo',
    }

    FAKE_ENV2 = {
        'UVC_HOST': '192.168.1.2',
        'UVC_PORT': '7443',
        'UVC_APIKEY': 'myKey',
    }

    @mock.patch('os.getenv')
    def test_get_auth_combined(self, mock_getenv):
        mock_getenv.side_effect = self.FAKE_ENV1.get
        host, port, key, path = nvr.get_auth_from_env()
        self.assertEqual('192.168.1.1', host)
        self.assertEqual(7080, port)
        self.assertEqual('foo', key)
        self.assertEqual('/', path)

    @mock.patch('os.getenv')
    def test_get_separate(self, mock_getenv):
        mock_getenv.side_effect = self.FAKE_ENV2.get
        host, port, key, path = nvr.get_auth_from_env()
        self.assertEqual('192.168.1.2', host)
        self.assertEqual(7443, port)
        self.assertEqual('myKey', key)
        self.assertEqual('/', path)



#!/usr/bin/env python
#
#   Copyright 2015 Dan Smith (dsmith+uvc@danplanet.com)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import logging
import pprint
import os
import sys
import zlib


# Python3 compatibility
try:
    import httplib
except ImportError:
    from http import client as httplib
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse


class Invalid(Exception):
    pass


class NotAuthorized(Exception):
    pass


class NvrError(Exception):
    pass


class UVCRemote(object):
    """Remote control client for Ubiquiti Unifi Video NVR."""
    CHANNEL_NAMES = ['high', 'medium', 'low']

    def __init__(self, host, port, apikey, path='/', ssl=False):
        self._host = host
        self._port = port
        self._path = path
        self._ssl = ssl
        if path != '/':
            raise Invalid('Path not supported yet')
        self._apikey = apikey
        self._log = logging.getLogger('UVC(%s:%s)' % (host, port))
        self._bootstrap = self._get_bootstrap()
        version = '.'.join(str(x) for x in self.server_version)
        self._log.debug('Server version is %s' % version)

    @property
    def server_version(self):
        version = self._bootstrap['systemInfo']['version'].split('.')
        major = int(version[0])
        minor = int(version[1])
        try:
            rev = int(version[2])
        except ValueError:
            rev = 0
        return (major, minor, rev)

    @property
    def camera_identifier(self):
        if self.server_version >= (3, 2, 0):
            return 'id'
        else:
            return 'uuid'
        
    def _get_http_connection(self):
        if self._ssl:
            return httplib.HTTPSConnection(self._host, self._port)
        else:
            return httplib.HTTPConnection(self._host, self._port)

    def _safe_request(self, *args, **kwargs):
        try:
            conn = self._get_http_connection();
            conn.request(*args, **kwargs)
            return conn.getresponse()
        except OSError:
            raise CameraConnectionError('Unable to contact camera')
        except httplib.HTTPException as ex:
            raise CameraConnectionError('Error connecting to camera: %s' % (
                str(ex)))

    def _uvc_request(self, *args, **kwargs):
        try:
            return self._uvc_request_safe(*args, **kwargs)
        except OSError:
            raise NvrError('Failed to contact NVR')
        except httplib.HTTPException as ex:
            raise NvrError('Error connecting to camera: %s' % str(ex))

    def _uvc_request_safe(self, path, method='GET', data=None,
                          mimetype='application/json'):
        conn = self._get_http_connection();
        if '?' in path:
            url = '%s&apiKey=%s' % (path, self._apikey)
        else:
            url = '%s?apiKey=%s' % (path, self._apikey)

        headers = {
            'Content-Type': mimetype,
            'Accept': 'application/json, text/javascript, */*; q=0.01',
            'Accept-Encoding': 'gzip, deflate, sdch',
        }
        self._log.debug('%s %s headers=%s data=%s' % (
            method, url, headers, repr(data)))
        conn.request(method, url, data, headers)
        resp = conn.getresponse()
        headers = dict(resp.getheaders())
        self._log.debug('%s %s Result: %s %s' % (method, url, resp.status,
                                                 resp.reason))
        if resp.status in (401, 403):
            raise NotAuthorized('NVR reported authorization failure')
        if resp.status / 100 != 2:
            raise NvrError('Request failed: %s' % resp.status)

        data = resp.read()
        if (headers.get('content-encoding') == 'gzip' or
                headers.get('Content-Encoding') == 'gzip'):
            data = zlib.decompress(data, 32 + zlib.MAX_WBITS)
        return json.loads(data.decode())

    def _get_bootstrap(self):
        return self._uvc_request('/api/2.0/bootstrap')['data'][0]

    def dump(self, uuid):
        """Dump information for a camera by UUID."""
        data = self._uvc_request('/api/2.0/camera/%s' % uuid)
        pprint.pprint(data)

    def set_recordmode(self, uuid, mode, chan=None):
        """Set the recording mode for a camera by UUID.

        :param uuid: Camera UUID
        :param mode: One of none, full, or motion
        :param chan: One of the values from CHANNEL_NAMES
        :returns: True if successful, False or None otherwise
        """

        url = '/api/2.0/camera/%s' % uuid
        data = self._uvc_request(url)
        settings = data['data'][0]['recordingSettings']
        mode = mode.lower()
        if mode == 'none':
            settings['fullTimeRecordEnabled'] = False
            settings['motionRecordEnabled'] = False
        elif mode == 'full':
            settings['fullTimeRecordEnabled'] = True
            settings['motionRecordEnabled'] = False
        elif mode == 'motion':
            settings['fullTimeRecordEnabled'] = False
            settings['motionRecordEnabled'] = True
        else:
            raise Invalid('Unknown mode')

        if chan:
            settings['channel'] = self.CHANNEL_NAMES.index(chan)
            changed = data['data'][0]['recordingSettings']

        data = self._uvc_request(url, 'PUT', json.dumps(data['data'][0]))
        updated = data['data'][0]['recordingSettings']
        return settings == updated

    def get_recordmode(self, uuid):
        url = '/api/2.0/camera/%s' % uuid
        data = self._uvc_request(url)
        recmodes = data['data'][0]['recordingSettings']
        if recmodes['fullTimeRecordEnabled']:
            return 'full'
        elif recmodes['motionRecordEnabled']:
            return 'motion'
        else:
            return 'none'

    def get_picture_settings(self, uuid):
        url = '/api/2.0/camera/%s' % uuid
        data = self._uvc_request(url)
        return data['data'][0]['ispSettings']

    def set_picture_settings(self, uuid, settings):
        url = '/api/2.0/camera/%s' % uuid
        data = self._uvc_request(url)
        for key in settings:
            dtype = type(data['data'][0]['ispSettings'][key])
            try:
                data['data'][0]['ispSettings'][key] = dtype(settings[key])
            except ValueError:
                raise Invalid('Setting `%s\' requires %s not %s' % (
                    key, dtype.__name__, type(settings[key]).__name__))
        data = self._uvc_request(url, 'PUT', json.dumps(data['data'][0]))
        return data['data'][0]['ispSettings']

    def prune_zones(self, uuid):
        url = '/api/2.0/camera/%s' % uuid
        data = self._uvc_request(url)
        data['data'][0]['zones'] = [data['data'][0]['zones'][0]]
        self._uvc_request(url, 'PUT', json.dumps(data['data'][0]))

    def list_zones(self, uuid):
        url = '/api/2.0/camera/%s' % uuid
        data = self._uvc_request(url)
        return data['data'][0]['zones']

    def index(self):
        """Return an index of available cameras.

        :returns: A list of dictionaries with keys of name, uuid
        """
        cams = self._uvc_request('/api/2.0/camera')['data']
        return [{'name': x['name'],
                 'uuid': x['uuid'],
                 'state': x['state'],
                 'managed': x['managed'],
                 'id': x['_id'],
             } for x in cams if not x['deleted']]

    def name_to_uuid(self, name):
        """Attempt to convert a camera name to its UUID.

        :param name: Camera name
        :returns: The UUID of the first camera with the same name if found,
                  otherwise None. On v3.2.0 and later, returns id.
        """
        cameras = self.index()
        if self.server_version >= (3, 2, 0):
            cams_by_name = {x['name']: x['id'] for x in cameras}
        else:
            cams_by_name = {x['name']: x['uuid'] for x in cameras}
        return cams_by_name.get(name)

    def get_camera(self, uuid):
        return self._uvc_request('/api/2.0/camera/%s' % uuid)['data'][0]

    def get_snapshot(self, uuid):
        url = '/api/2.0/snapshot/camera/%s?force=true&apiKey=%s' % (
            uuid, self._apikey)
        print(url)
        resp = self._safe_request('GET', url)
        if resp.status != 200:
            raise NvrError('Snapshot returned %i' % resp.status)
        return resp.read()


def get_auth_from_env():
    """Attempt to get UVC NVR connection information from the environment.

    Supports either a combined variable called UVC formatted like:

        UVC="http://192.168.1.1:7080/?apiKey=XXXXXXXX"

    or individual ones like:

        UVC_HOST=192.168.1.1
        UVC_PORT=7080
        UVC_APIKEY=XXXXXXXXXX

    :returns: A tuple like (host, port, apikey, path)
    """

    combined = os.getenv('UVC')
    if combined:
        # http://192.168.1.1:7080/apikey
        result = urlparse.urlparse(combined)
        if ':' in result.netloc:
            host, port = result.netloc.split(':', 1)
            port = int(port)
        else:
            host = result.netloc
            port = 7080
        apikey = urlparse.parse_qs(result.query)['apiKey'][0]
        path = result.path
    else:
        host = os.getenv('UVC_HOST')
        port = int(os.getenv('UVC_PORT', 7080))
        apikey = os.getenv('UVC_APIKEY')
        path = '/'
    return host, port, apikey, path



from setuptools import setup

setup(name='uvcclient',
      version='0.11.1',
      description='A remote control client for Ubiquiti\'s UVC NVR',
      author='Dan Smith',
      author_email='dsmith+uvcclient@danplanet.com',
      url='http://github.org/kk7ds/uvcclient',
      packages=['uvcclient'],
      scripts=['uvc'],
      install_requires=[],
      tests_require=['mock'],
      classifiers=[
            "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
      ]
)



try:
    import httplib
except ImportError:
    from http import client as httplib

import json
import unittest

import mock

from uvcclient import camera


class TestCamera(unittest.TestCase):
    def test_set_led_on(self):
        c = camera.UVCCameraClient('foo', 'ubnt', 'ubnt')
        with mock.patch.object(c, '_cfgwrite') as mock_write:
            c.set_led(False)
            mock_write.assert_called_once_with('led.front.status', 0)

    def test_set_led_off(self):
        c = camera.UVCCameraClient('foo', 'ubnt', 'ubnt')
        with mock.patch.object(c, '_cfgwrite') as mock_write:
            c.set_led(True)
            mock_write.assert_called_once_with('led.front.status', 1)

    def test_get_snapshot(self):
        c = camera.UVCCameraClient('foo', 'ubnt', 'ubnt')
        with mock.patch.object(httplib, 'HTTPConnection') as mock_conn:
            conn = mock_conn.return_value
            conn.getresponse.return_value.status = 200
            r = c.get_snapshot()
            self.assertEquals(conn.getresponse.return_value.read.return_value,
                              r)

    def test_cfgwrite(self):
        c = camera.UVCCameraClient('foo', 'ubnt', 'ubnt')
        c._cookie = 'foo-cookie'
        with mock.patch.object(httplib, 'HTTPConnection') as mock_h:
            conn = mock_h.return_value
            conn.getresponse.return_value.status = 200
            self.assertTrue(c._cfgwrite('foo', 'bar'))
            headers = {'Cookie': 'foo-cookie'}
            conn.request.assert_called_once_with('GET',
                                                 '/cfgwrite.cgi?foo=bar',
                                                 headers=headers)

    @mock.patch.object(httplib, 'HTTPConnection')
    def test_login_v310(self, mock_h):
        c = camera.UVCCameraClient('foo', 'ubnt', 'ubnt')
        first = mock.MagicMock()
        second = mock.MagicMock()
        counter = [0]

        def fake_conn(*a, **k):
            if counter[0] == 0:
                counter[0] += 1
                return first
            elif counter[0] == 1:
                counter[0] += 1
                return second

        mock_h.side_effect = fake_conn
        cookie = 'thecookie AIROS_SESSIONID=foo; bar'
        first.getresponse.return_value.getheaders.return_value = [
            ('Set-Cookie', cookie)]
        second.getresponse.return_value.status = 200
        c.login()
        first.request.assert_called_once_with('GET', '/')
        self.assertEqual('POST',
                         second.request.call_args_list[0][0][0])
        self.assertEqual('/login.cgi',
                         second.request.call_args_list[0][0][1])
        formdata = 'AIROS_SESSIONID=foo&password=ubnt&username=ubnt'
        self.assertEqual(
            sorted(formdata.split('&')),
            sorted(second.request.call_args_list[0][0][2].split('&')))
        self.assertEqual(cookie,
                         second.request.call_args_list[0][0][3]['Cookie'])

    @mock.patch.object(httplib, 'HTTPConnection')
    def test_login_v320(self, mock_h):
        c = camera.UVCCameraClientV320('foo', 'ubnt', 'ubnt')
        resp = mock_h.return_value.getresponse.return_value
        resp.status = 200
        resp.getheaders.return_value = {'set-cookie': 'cookie'}
        c.login()
        self.assertEqual('POST',
                         mock_h.return_value.request.call_args_list[0][0][0])
        self.assertEqual('/api/1.1/login',
                         mock_h.return_value.request.call_args_list[0][0][1])
        data = json.loads(mock_h.return_value.request.call_args_list[0][0][2])
        self.assertEqual({'username': 'ubnt', 'password': 'ubnt'},
                         data)
        self.assertEqual('cookie', c._cookie)



#   Copyright 2015 Dan Smith (dsmith+uvc@danplanet.com)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import socket

# Python3 compatibility
try:
    import httplib
except ImportError:
    from http import client as httplib
try:
    import urlparse
    import urllib
except ImportError:
    import urllib.parse as urlparse


class CameraConnectError(Exception):
    pass


class CameraAuthError(Exception):
    pass


class UVCCameraClient(object):
    def __init__(self, host, username, password, port=80):
        self._host = host
        self._port = port
        self._username = username
        self._password = password
        self._cookie = ''
        self._log = logging.getLogger('UVCCamera(%s)' % self._host)

    def _safe_request(self, *args, **kwargs):
        try:
            conn = httplib.HTTPConnection(self._host, self._port)
            conn.request(*args, **kwargs)
            return conn.getresponse()
        except (socket.error, OSError):
            raise CameraConnectError('Unable to contact camera')
        except httplib.HTTPException as ex:
            raise CameraConnectError('Error connecting to camera: %s' % (
                str(ex)))

    def login(self):
        headers = {'Content-Type': 'application/json'}
        data = json.dumps({'username': self._username,
                           'password': self._password})
        resp = self._safe_request('POST', '/api/1.1/login', data,
                                  headers=headers)
        if resp.status != 200:
            raise CameraAuthError('Failed to login: %s' % resp.reason)
        headers = dict(resp.getheaders())
        try:
            self._cookie = headers['Set-Cookie']
        except KeyError:
            self._cookie = headers['set-cookie']

    def _cfgwrite(self, setting, value):
        headers = {'Cookie': self._cookie}
        resp = self._safe_request(
            'GET', '/cfgwrite.cgi?%s=%s' % (setting, value),
            headers=headers)
        self._log.debug('Setting %s=%s: %s %s' % (setting, value,
                                                  resp.status,
                                                  resp.reason))
        return resp.status == 200

    def set_led(self, enabled):
        return self._cfgwrite('led.front.status', int(enabled))

    def get_snapshot(self):
        conn = httplib.HTTPConnection(self._host, self._port)
        headers = {'Cookie': self._cookie}
        resp = self._safe_request('GET', '/snap.jpeg',
                                  headers=headers)
        if resp.status in (401, 403, 302):
            raise CameraAuthError('Not logged in')
        elif resp.status != 200:
            raise CameraConnectError(
                'Snapshot failed: %s' % resp.status)
        return resp.read()
